<?php

/** @mainpage RDF Relationship Packages
 *
 * 
 * The Relationships API - where I try to logically say 
 * all things that can be said about a Drupal Node ... in terms of RDF.
 * 
 * - Components of The Relationships package
 *   - @link relationship_api API - The interface to set and get node relationships and metadata  @endlink
 *   - @link ontologies Ontologies - Pluggable sets of terms and types of relationships @endlink 
 *   - @link docs Docs - Theory and background documentation @endlink 
 */ 


/**
 * @defgroup docs Theory and background documentation
 *
 * Dear coder
 *
 * If you are looking at this code, here's a few explanations
 *
 * The function names are long - I tend towards trying to make code make
 * sense when read out loud.
 * Same goes for the variable names.
 *
 * This package looks big - Actually 2/3 (or more) is comments. There's a lot
 * of concepts I wrote about as I went, re-writing or clarifying as the code
 * developed rather than in a separate documentation file. I expected large
 * amounts of this to be pulled out by a doc-gen thing, but doxygen is not
 * co-operating, and I haven't been able to test the output very well. As such 
 * there may be a few inconsistancies with my @ tags.
 *
 * In many cases I use three statements where one line would do. Usually that
 * is to explicitly name an intermediate variable to help with conceptual 
 * understanding - for me. Or to dump debug output in the meantime.
 * But some things may be ready to be made simpler again.
 * The ternary op is cool to write, but always hard to read.
 *
 * I've got lots of (transient) caching going on. It may be excessive, and 
 * could even cause some head-scratching. 
 * It does I guess increase memory requirements a bit - it's a trade-off.
 * Even with logical recursion through statements going on, I'm hoping to be
 * dealing with dozens, not thousands of triples 
 * at a time. ... to be investigated.
 *
 * Turning up the DEBUGLEVEL below will dump lots of info.
 * Comments marked out with # are serious scaffolding - expected to be 
 * removed when I'm happy with the rest of it.
 *
 * Any suggestions on fixing up the coding structure and making the API
 * doc reader work better are welcome.
 *
 * .dan.
 */
/**
 * @{
 */

/** 
 * @defgroup drupal_hooks Drupal Hooks
 * 
 * Nothing exciting to say about these functions.
 *
 * @link http://drupaldocs.org/api/4.6/group/hooks See the drupal Docs for more detail @endlink
 */ 

/** 
 * @defgroup relationship_api API
 * 
 * User/Developer guide to the public methods available in the package.
 *
 * All non-private functions are available to use as utilities, but the functions 
 * listed here as API are explicitly the face of this package, here for other
 * modules to use.
 */ 



/** 
 * @defgroup predicates Predicates
 *
 * About how predicate terms are managed internally.
 *
 * All predicates may optionally have characteristics. It's the characteristics
 * which give each predicate term its logical structure.
 *
 * A predicate object may optionally include a few defining 'characteristics'
 * such as its domain, its range, its ordinality constraints, inverse or
 * transitive properties. These settings add logical structure to the meaning of
 * each term. This extra meaning makes it an 'ontology', which is like a
 * 'taxonomy' but with added logic.
 *
 * Internally, predicate definitions are retrieved using relationship_get_term()
 * and are handed around as small objects.
 *
 * A sample term may look like:
 *     $parent_def = stdClass(
 *      'name' => 'parentOf',
 *      'uri' => 'http://purl.org/vocab/relationship/parentOf',
 *      'label' => 'Parent Of',
 *      'definition' => 'A person who has given birth to or nurtured and raised this person" ',
 *      'weight' => '2',
 *      'inverseOf' => 'childOf'
 *     )   
 * 
 * Internally, a term is managed as a collection of statements, existing only
 * through the confluence of right sort of information. There is no table and
 * there is no row that defines a term.
 * The statements 
 * 
 * 'nav:parentOf' 'rdf:isType'    'rdfs:Property'
 * 'nav:parentOf' 'rdf:label'     'Parent Of'
 * 'nav:parentOf' 'owl:inverseOf' 'nav:childOf'
 * 
 * ... and more ...
 * come together to tell us what we need to know about the 'parentOf' term.
 *
 * A predicate has a URI as a unique identifier, but for convenience we use
 * only its shortname most of the time. 
 * A statement:
 * 'http://purl.org/vocab/relationship/parentOf' : 'identifier' : 'parentOf'
 * records this. (the identifier of {long URL} is {short name})
 *
 * Internally, the term definitions are heavily cached (as they are used a lot).
 * the 'shortnames' of all property terms are stored in drupal variables, and
 * the term defs themselves are kept in the cache, refreshed from the database
 * as needed - whenever a term is saved. If they get out of date, or a
 * change goes undetected, visiting the predicate admin pages should flush it.
 *
 * @see administer_predicates.inc
 *
 * @todo 
 *  My 'characteristics' based on my reading of W3C OWL should actually
 *  be broken down into owl:restriction and  owl:objectProperty ?
 *
 * @todo 
 *  I need to see how and if to use namespaces internally.
 *  Use the RDF isDefinedBy or what?
 *
 * @ingroup docs 
 *
 */

 
/**
 * @defgroup extensions Extending the module
 *
 * Modules may supply their own terms, their own logic and even their own
 * ontology characteristics.
 *
 * An Ontology is a set of meanings. 
 * Defining an ontology plugin module means defining some terms and what they actually do.
 *
 * By declaring a set of custom predicates, new terms can be applied to node
 * relationships. 
 * 
 * By giving the predicates characteristics, new logical
 * behaviours can support the terms. 
 * 
 * By using the usual nodeapi or blocks, the
 * new relationships can be used to change the way pages are presented.
 * 
 * 
 * @see ontology_navigation.module
 * @see ontology_revision_control.module
 *
 * @ingroup docs Theory and background documentation
 */


/** 
 * @defgroup ontologies Ontologies
 *
 * == Groups of logical predicate terms. ==
 *
 * An overview of Ontologies, as implimented by the relationship.module.
 *
 * === About Extending the module, defining your own predicates ontologies and
 * behaviours. ===
 *
 * Modules may supply their own terms,  their own logic and even their own
 * ontology characteristics.
 *
 * An Ontology is a set of meanings. 
 * Defining an ontology plugin module means defining some terms and what they
 * actually do.
 *
 * By declaring a set of custom predicates, new terms can be applied to node
 * relationship.
 * 
 * By giving the predicates characteristics, new logical behaviours can support
 * the terms. A set of relationship definitions could be used to create a new
 * way or threading conversations in discussion groups by defining 'IsReplyTo'
 * and 'IsInThread' then extending it to 'IsRebuttalOf' and 'IsDigression' to
 * see what happens.
 * <http://www.eyrie.org/~zednenem/2002/web-threads/>
 *
 * By using the usual nodeapi or blocks,  the new relationship
 * can be used to change the way pages are presented. For example
 * by providing more than one flavour of 'Next' button.
 * Next in thread,  next in time,  etc
 *
 * == Roadmap ==
 * 
 * Overview on how to start developiong an extension ontology module.
 * - Choose the terms you wish to use, they can be added to or refined at any
 * stage.
 * 
 * - Define these terms as 'Properties' using the relationship interface.
 * 
 * - Add 'domain' and 'range' restrictions to your new term, to state what types
 * of things it's supposed to link together. 
 * 
 * - For example, if your new term has a 'domain' of drupal:node, from now on
 * editing any drupal node should display a way to create a relationship using
 * that term.
 * 
 * - Export your new terms into an RDF schema dump. This is the new ontology
 * that relates to your project, and can be the starting point for the new
 * ontology module. 
 * 
 * - A common next step for an ontology module would be to
 * define a way to render the new relationship. This could be done by
 * implimenting hook_links() or hook_nodeapi ('view') to show the relationships
 * on the page when the node is viewed. 
 * 
 * - Normally, the relationships will be accessable after node_load as a $node-
 * >relationships array. If you create a drupalEquivalent mapping characteristic
 * for your new term, the value of the relationship will be imported into the
 * node object onload, and be visible as '$node->congruent' or whatever.
 * 
 * - To make advanced lookups, the relationship_api provides a
 * relationship_query function. This also allows custom callbacks to be
 * implimented for your custom term. eg, if your new relationship is
 * 'congruent', then when any lookup is made, "what is congruent to node 55?" as
 * well as any suitable statements being returned, the callback hook
 * relationship_lookup_congruent() is also called, and if implimented, you can
 * return your own custom answers to that query.
 * 
 * - You may also want to provide a better interface to set and edit the new
 * type of relationship. This can be done with some work in hook_form_alter, and
 * with reference to the relationship_api functions.
 * 
 * 
 *
 * @see relationship_rdf.module to import a whole ontology from public RDF
 * schemas.
 * @see ontology_forum.module for an experiment in using relationships to represent a forum.
 * @see ontology_navigation.module as an example
 * @see ontology_geneology.module as an example of making your own schema.
 * @ingroup docs 
 */


/**
 * A callback used to return the result of requesting a specific predicate
 * 
 * Implimented by custom ontologies that need extra logic to calculate their
 * response. The relationship_api will invoke this as part of its response to a
 * query, for example, to retrieve a list of all nodes more recent than the
 * current one.
 * 
 * $statements = relationship_query('this_item','olderThan','');
 * 
 * (A list of resources that this one is OlderThen)
 * 
 *  This should return both the
 * list of results of the static query (if any) and in this case also requires a
 * dynamic lookup, as this result is likely to change without the datastore
 * knowing.
 * 
 * @param $subject The URI or identifier of the subject
 * @param $query optional, the full query being made
 * @return array of statements. MUST return an array, even if empty.
 */
function relationship_lookup_hook($subject,$query=array()){
  $resource = relationship_get_resource($subject);
  $result = db_query("SELECT * FROM {node} WHERE changed > %n",$resource->changed);
  $statements = array();
  while($new = db_get_object()){
    // Construct the response as a set of statements
    $statements[] = relationship_statement($subject,'olderThan',$new->path,NULL,0,'Calculated from changed dates');  
    // Providing a 'why' may not be neccessary, but helps for tracing debug info and trust.
  }
  return $statements;
}

/**
 * Concepts
 * 
 * == Namespaces ==
 * - What's this URL http://drupal.org/project/relationship#canUseOntology etc?.
 * Firstly, this is NOT an URL, it's a string that looks like an URL. 
 * It's just a unique identifier that means something special, not only in this
 * contenxt, but to all systems that speak the same language.
 * Confusingly, the one thing it DOESN'T mean is that there is anything useful 
 * to be found by visiting the URL. It's not designed to be resolved!.
 * I used a namespace underneath drupal.org/project/relationship to hint that
 * some more information can be found there - but it's not required that this
 * information is machine-readable (unlike a DTD or something).
 * If I COULD find a nice place to publish the 'Drupal Schema', we could use
 * that instead.
 *
 * == The 'Drupal Schema' is likewise mostly fake. ==
 * The Drupal concept of a 'node' and a 'content type' are unique to the Drupal 
 * project. Yet they have an internal logic that can be expressed via 
 * relationships, and to express those relationships we use a specialised
 * terminology. Thus, when a statement is made about the resource identified
 * as "http://drupal.org/project/relationship#node" , we are talking about
 * Drupal nodes, not graph nodes or anything else. 
 * I've made up the terms as needed, and some of them could probably be
 * re-worked, depending on how they are to be used.
 * The shorthand for these terms will be assumed to be the namespace prefix
 * 'drupal:'  
 *
 * == DrupalEquivalent Mapping ==
 * A direct mapping between some internal Drupal terms is also defined.
 * The Drupal node 'title' - the string found as the 'title' property on
 * a drupal:node is equivalent to the Dublin core definition of what a title
 * is, so 
 *   dc:title drupal:drupalEquivalent drupal:title
 * However, the Drupal node 'name' property translates better to the dc:creator
 * property, and drupal:type ... doesn't translate well at all.
 
 */

/**
 * Executes a custom lookup for a predicate query.
 *
 * Prototype function - modules can declare their own version
 * to extend the predicate vocabulary. Sample code here is only
 * for demonstration!
 *
 * When a module defines a new predicate term, it may also define the 
 * way in which a query on that predicate is resolved.
 * This can either be done by assigning the term some 'characteristics'
 * and letting an inference engine resolve the result, or by implimenting
 * this relationship_lookup_hook() and calculating the query itself.
 *
 * In simple cases, this hook need not be implimented at all.
 * Basic cases (like reciprocal lookups) are best handled using characteristics.
 * But if you need to combine iteration with recursion (like listing all
 * second cousins) the logic can be custom-coded here.
 *
 * Pseudo-parameter predicate_label replaces the word 'hook' in the function name,
 * eg relationship_lookup_hasCousin('Enoch')
 *
 * This function sttempts to return an array containing the statement(s)
 * claiming that something has the stated relationship to the current item.
 * A relationship_lookup_hook() will always return an ARRAY of full STATEMENTS.
 *
 * @param $subject the subject identifier the query is run on. May be null.
 * @param $max_recursion_level
 *
 * @returns an array of statements answering the query: { ?subject ?predicate_name ?}
 * @see relationship_query()
 * @ingroup docs 
 * @ingroup ontologies
 */
function relationship_lookup_hook($subject,  $max_recursion_level=1,  $depth=0) {

  // Demo Code
  // This is just illustrating how you MAY wish
  // to retrieve all 'cousins' of a subject, assuming
  // the hasUncle and ParentOf predicated are also 
  // declared somewhere.

  $statements = array(); 
  
  $uncles = relationship_get_properties($subject, 'hasUncle', $max_recursion_level);
  
  foreach($uncles as $uncle){
    $uncles_children = relationship_get_properties($uncle, 'ParentOf', $max_recursion_level);
    
    foreach($uncles_children as $cousin){
      $statements[] = relationship_statement($subject,  'hasCousin',  $cousin);
    }
  }
  
  // get_titles validates and fleshes-out the statements
  relationship_get_titles($statements);

  return $statements;
  // Always return an array. It could be empty, but that's an informative response.
}


/**
 * @}
 */
